@@ -591,7 +591,7 @@ class ApiController(base.BaseController):
             limit = 20
         limit = min(50, limit)
 
-        query = model.Group.search_by_name(q, t)
+        query = model.Group.search_by_name_or_title(q, t)
         def convert_to_dict(user):
             out = {}
             for k in ['id', 'name', 'title']:
