@@ -1,37 +1,42 @@
-from pylons import config
-from sqlalchemy.sql import select, and_
 import datetime
-
-from ckan.model import PackageRevision
-from ckan.lib.dictization import (obj_list_dictize,
-                                  obj_dict_dictize,
-                                  table_dictize)
-from ckan.logic import NotFound
+from pylons import config
+from sqlalchemy.sql import select
+import datetime
+import ckan.model
 import ckan.misc
-from ckan.lib.helpers import json
+import ckan.logic as logic
+import ckan.plugins as plugins
+import ckan.lib.helpers as h
+import ckan.lib.dictization as d
 
 ## package save
 
-def group_list_dictize(obj_list, context, 
+def group_list_dictize(obj_list, context,
                        sort_key=lambda x:x['display_name'], reverse=False):
 
     active = context.get('active', True)
-
+    with_private = context.get('include_private_packages', False)
     result_list = []
 
     for obj in obj_list:
         if context.get('with_capacity'):
             obj, capacity = obj
-            group_dict = table_dictize(obj, context, capacity=capacity)
+            group_dict = d.table_dictize(obj, context, capacity=capacity)
         else:
-            group_dict = table_dictize(obj, context)
+            group_dict = d.table_dictize(obj, context)
         group_dict.pop('created')
         if active and obj.state not in ('active', 'pending'):
             continue
 
         group_dict['display_name'] = obj.display_name
 
-        group_dict['packages'] = len(obj.active_packages().all())
+        group_dict['packages'] = \
+                len(obj.active_packages(with_private=with_private).all())
+
+        if context.get('for_view'):
+            for item in plugins.PluginImplementations(
+                    plugins.IGroupController):
+                group_dict = item.before_view(group_dict)
 
         result_list.append(group_dict)
     return sorted(result_list, key=sort_key, reverse=reverse)
@@ -41,23 +46,34 @@ def resource_list_dictize(res_list, context):
     active = context.get('active', True)
     result_list = []
     for res in res_list:
+        resource_dict = resource_dictize(res, context)
         if active and res.state not in ('active', 'pending'):
             continue
-        result_list.append(resource_dictize(res, context))
+
+        result_list.append(resource_dict)
 
     return sorted(result_list, key=lambda x: x["position"])
 
+def related_list_dictize(related_list, context):
+    result_list = []
+    for res in related_list:
+        related_dict = related_dictize(res, context)
+        result_list.append(related_dict)
+
+    return sorted(result_list, key=lambda x: x["created"], reverse=True)
+
+
 def extras_dict_dictize(extras_dict, context):
     result_list = []
     for name, extra in extras_dict.iteritems():
-        dictized = table_dictize(extra, context)
+        dictized = d.table_dictize(extra, context)
         if not extra.state == 'active':
             continue
         value = dictized["value"]
         ## This is to make sure the frontend does not show a plain string
         ## as json with brackets.
         if not(context.get("extras_as_string") and isinstance(value, basestring)):
-            dictized["value"] = json.dumps(value)
+            dictized["value"] = h.json.dumps(value)
         result_list.append(dictized)
 
     return sorted(result_list, key=lambda x: x["key"])
@@ -66,23 +82,31 @@ def extras_list_dictize(extras_list, context):
     result_list = []
     active = context.get('active', True)
     for extra in extras_list:
+        dictized = d.table_dictize(extra, context)
         if active and extra.state not in ('active', 'pending'):
             continue
-        dictized = table_dictize(extra, context)
         value = dictized["value"]
         if not(context.get("extras_as_string") and isinstance(value, basestring)):
-            dictized["value"] = json.dumps(value)
+            dictized["value"] = h.json.dumps(value)
         result_list.append(dictized)
 
     return sorted(result_list, key=lambda x: x["key"])
 
 def resource_dictize(res, context):
-    resource = table_dictize(res, context)
+    resource = d.table_dictize(res, context)
     extras = resource.pop("extras", None)
     if extras:
         resource.update(extras)
+    #tracking
+    if not context.get('for_edit'):
+        model = context['model']
+        tracking = model.TrackingSummary.get_for_resource(res.url)
+        resource['tracking_summary'] = tracking
     return resource
 
+def related_dictize(rel, context):
+    return d.table_dictize(rel, context)
+
 def _execute_with_revision(q, rev_table, context):
     '''
     Takes an SqlAlchemy query (q) that is (at its base) a Select on an
@@ -93,10 +117,10 @@ def _execute_with_revision(q, rev_table, context):
     But you can provide revision_id, revision_date or pending in the
     context and it will filter to an earlier time or the latest unmoderated
     object revision.
-    
+
     Raises NotFound if context['revision_id'] is provided, but the revision
     ID does not exist.
-    
+
     Returns [] if there are no results.
 
     '''
@@ -111,9 +135,9 @@ def _execute_with_revision(q, rev_table, context):
         revision = session.query(context['model'].Revision).filter_by(
             id=revision_id).first()
         if not revision:
-            raise NotFound
+            raise logic.NotFound
         revision_date = revision.timestamp
-    
+
     if revision_date:
         q = q.where(rev_table.c.revision_timestamp <= revision_date)
         q = q.where(rev_table.c.expired_timestamp > revision_date)
@@ -133,7 +157,7 @@ def package_dictize(pkg, context):
     but you can provide revision_id, revision_date or pending in the
     context and it will filter to an earlier time or the latest unmoderated
     object revision.
-    
+
     May raise NotFound. TODO: understand what the specific set of
     circumstances are that cause this.
     '''
@@ -143,49 +167,62 @@ def package_dictize(pkg, context):
     q = select([package_rev]).where(package_rev.c.id == pkg.id)
     result = _execute_with_revision(q, package_rev, context).first()
     if not result:
-        raise NotFound
-    result_dict = table_dictize(result, context)
+        raise logic.NotFound
+    result_dict = d.table_dictize(result, context)
     #resources
     res_rev = model.resource_revision_table
     resource_group = model.resource_group_table
-    q = select([res_rev], from_obj = res_rev.join(resource_group, 
+    q = select([res_rev], from_obj = res_rev.join(resource_group,
                resource_group.c.id == res_rev.c.resource_group_id))
     q = q.where(resource_group.c.package_id == pkg.id)
     result = _execute_with_revision(q, res_rev, context)
     result_dict["resources"] = resource_list_dictize(result, context)
+
     #tags
     tag_rev = model.package_tag_revision_table
     tag = model.tag_table
-    q = select([tag, tag_rev.c.state, tag_rev.c.revision_timestamp], 
+    q = select([tag, tag_rev.c.state, tag_rev.c.revision_timestamp],
         from_obj=tag_rev.join(tag, tag.c.id == tag_rev.c.tag_id)
         ).where(tag_rev.c.package_id == pkg.id)
     result = _execute_with_revision(q, tag_rev, context)
-    result_dict["tags"] = obj_list_dictize(result, context, lambda x: x["name"])
+    result_dict["tags"] = d.obj_list_dictize(result, context, lambda x: x["name"])
+
+    # Add display_names to tags. At first a tag's display_name is just the
+    # same as its name, but the display_name might get changed later (e.g.
+    # translated into another language by the multilingual extension).
+    for tag in result_dict['tags']:
+        assert not tag.has_key('display_name')
+        tag['display_name'] = tag['name']
+
     #extras
     extra_rev = model.extra_revision_table
     q = select([extra_rev]).where(extra_rev.c.package_id == pkg.id)
     result = _execute_with_revision(q, extra_rev, context)
     result_dict["extras"] = extras_list_dictize(result, context)
+    #tracking
+    tracking = model.TrackingSummary.get_for_package(pkg.id)
+    result_dict['tracking_summary'] = tracking
     #groups
     member_rev = model.member_revision_table
     group = model.group_table
-    q = select([group],
+    q = select([group, member_rev.c.capacity],
                from_obj=member_rev.join(group, group.c.id == member_rev.c.group_id)
-               ).where(member_rev.c.table_id == pkg.id)
+               ).where(member_rev.c.table_id == pkg.id)\
+                .where(member_rev.c.state == 'active')
     result = _execute_with_revision(q, member_rev, context)
-    result_dict["groups"] = obj_list_dictize(result, context)
+    result_dict["groups"] = d.obj_list_dictize(result, context)
     #relations
     rel_rev = model.package_relationship_revision_table
     q = select([rel_rev]).where(rel_rev.c.subject_package_id == pkg.id)
     result = _execute_with_revision(q, rel_rev, context)
-    result_dict["relationships_as_subject"] = obj_list_dictize(result, context)
+    result_dict["relationships_as_subject"] = d.obj_list_dictize(result, context)
     q = select([rel_rev]).where(rel_rev.c.object_package_id == pkg.id)
     result = _execute_with_revision(q, rel_rev, context)
-    result_dict["relationships_as_object"] = obj_list_dictize(result, context)
-    
+    result_dict["relationships_as_object"] = d.obj_list_dictize(result, context)
+
     # Extra properties from the domain object
     # We need an actual Package object for this, not a PackageRevision
-    if isinstance(pkg,PackageRevision):
+    if isinstance(pkg, ckan.model.PackageRevision):
         pkg = model.Package.get(pkg.id)
 
     # isopen
@@ -194,12 +231,24 @@ def package_dictize(pkg, context):
     # type
     result_dict['type']= pkg.type
 
+    # licence
+    if pkg.license and pkg.license.url:
+        result_dict['license_url']= pkg.license.url
+        result_dict['license_title']= pkg.license.title.split('::')[-1]
+    elif pkg.license:
+        result_dict['license_title']= pkg.license.title
+    else:
+        result_dict['license_title']= pkg.license_id
+
     # creation and modification date
-    result_dict['metadata_modified'] = pkg.metadata_modified.isoformat() \
-        if pkg.metadata_modified else None
+    result_dict['metadata_modified'] = context.pop('metadata_modified')
     result_dict['metadata_created'] = pkg.metadata_created.isoformat() \
         if pkg.metadata_created else None
 
+    if context.get('for_view'):
+        for item in plugins.PluginImplementations( plugins.IPackageController):
+            result_dict = item.before_view(result_dict)
+
     return result_dict
 
 def _get_members(context, group, member_type):
@@ -212,10 +261,9 @@ def _get_members(context, group, member_type):
                filter(model.Member.state == 'active').\
                filter(model.Member.table_name == member_type[:-1]).all()
 
-
 def group_dictize(group, context):
     model = context['model']
-    result_dict = table_dictize(group, context)
+    result_dict = d.table_dictize(group, context)
 
     result_dict['display_name'] = group.display_name
 
@@ -224,7 +272,7 @@ def group_dictize(group, context):
 
     context['with_capacity'] = True
 
-    result_dict['packages'] = obj_list_dictize(
+    result_dict['packages'] = d.obj_list_dictize(
         _get_members(context, group, 'packages'),
         context)
 
@@ -242,6 +290,10 @@ def group_dictize(group, context):
 
     context['with_capacity'] = False
 
+    if context.get('for_view'):
+        for item in plugins.PluginImplementations(plugins.IGroupController):
+            result_dict = item.before_view(result_dict)
+
     return result_dict
 
 def tag_list_dictize(tag_list, context):
@@ -250,23 +302,32 @@ def tag_list_dictize(tag_list, context):
     for tag in tag_list:
         if context.get('with_capacity'):
             tag, capacity = tag
-            dictized = table_dictize(tag, context, capacity=capacity)
+            dictized = d.table_dictize(tag, context, capacity=capacity)
         else:
-            dictized = table_dictize(tag, context)
+            dictized = d.table_dictize(tag, context)
         result_list.append(dictized)
 
     return result_list
 
 def tag_dictize(tag, context):
 
-    result_dict = table_dictize(tag, context)
+    result_dict = d.table_dictize(tag, context)
+    result_dict["packages"] = d.obj_list_dictize(tag.packages, context)
 
-    result_dict["packages"] = obj_list_dictize(
-        tag.packages_ordered, context)
-    
-    return result_dict 
+    # Add display_names to tags. At first a tag's display_name is just the
+    # same as its name, but the display_name might get changed later (e.g.
+    # translated into another language by the multilingual extension).
+    assert not result_dict.has_key('display_name')
+    result_dict['display_name'] = result_dict['name']
 
-def user_list_dictize(obj_list, context, 
+    if context.get('for_view'):
+        for item in plugins.PluginImplementations(
+                plugins.ITagController):
+            result_dict = item.before_view(result_dict)
+
+    return result_dict
+
+def user_list_dictize(obj_list, context,
                       sort_key=lambda x:x['name'], reverse=False):
 
     result_list = []
@@ -277,54 +338,52 @@ def user_list_dictize(obj_list, context,
         result_list.append(user_dict)
     return sorted(result_list, key=sort_key, reverse=reverse)
 
+def member_dictize(member, context):
+    return d.table_dictize(member, context)
 
 def user_dictize(user, context):
 
     if context.get('with_capacity'):
         user, capacity = user
-        result_dict = table_dictize(user, context, capacity=capacity)
+        result_dict = d.table_dictize(user, context, capacity=capacity)
     else:
-        result_dict = table_dictize(user, context)
+        result_dict = d.table_dictize(user, context)
 
     del result_dict['password']
-    
+
     result_dict['display_name'] = user.display_name
     result_dict['email_hash'] = user.email_hash
     result_dict['number_of_edits'] = user.number_of_edits()
     result_dict['number_administered_packages'] = user.number_administered_packages()
 
-    return result_dict 
+    return result_dict
 
 def task_status_dictize(task_status, context):
-    return table_dictize(task_status, context)
+    return d.table_dictize(task_status, context)
 
 ## conversion to api
 
-def group_to_api1(group, context):
-    
+def group_to_api(group, context):
+    api_version = context.get('api_version')
+    assert api_version, 'No api_version supplied in context'
     dictized = group_dictize(group, context)
-    dictized["extras"] = dict((extra["key"], json.loads(extra["value"])) 
+    dictized["extras"] = dict((extra["key"], h.json.loads(extra["value"]))
                               for extra in dictized["extras"])
-    dictized["packages"] = sorted([package["name"] for package in dictized["packages"]])
+    if api_version == 1:
+        dictized["packages"] = sorted([pkg["name"] for pkg in dictized["packages"]])
+    else:
+        dictized["packages"] = sorted([pkg["id"] for pkg in dictized["packages"]])
     return dictized
 
-def group_to_api2(group, context):
-    
-    dictized = group_dictize(group, context)
-    dictized["extras"] = dict((extra["key"], json.loads(extra["value"])) 
-                              for extra in dictized["extras"])
-    dictized["packages"] = sorted([package["id"] for package in dictized["packages"]])
-    return dictized
-
-def tag_to_api1(tag, context):
-    
+def tag_to_api(tag, context):
+    api_version = context.get('api_version')
+    assert api_version, 'No api_version supplied in context'
     dictized = tag_dictize(tag, context)
-    return sorted([package["name"] for package in dictized["packages"]])
+    if api_version == 1:
+        return sorted([package["name"] for package in dictized["packages"]])
+    else:
+        return sorted([package["id"] for package in dictized["packages"]])
 
-def tag_to_api2(tag, context):
-
-    dictized = tag_dictize(tag, context)
-    return sorted([package["id"] for package in dictized["packages"]])
 
 def resource_dict_to_api(res_dict, package_id, context):
     res_dict.pop("revision_id")
@@ -333,132 +392,118 @@ def resource_dict_to_api(res_dict, package_id, context):
     res_dict["package_id"] = package_id
 
 
-def package_to_api1(pkg, context):
-
+def package_to_api(pkg, context):
+    api_version = context.get('api_version')
+    assert api_version, 'No api_version supplied in context'
     dictized = package_dictize(pkg, context)
-
     dictized.pop("revision_timestamp")
 
-    dictized["groups"] = [group["name"] for group in dictized["groups"]]
-    dictized["tags"] = [tag["name"] for tag in dictized["tags"]]
-    dictized["extras"] = dict((extra["key"], json.loads(extra["value"])) 
+    dictized["tags"] = [tag["name"] for tag in dictized["tags"] \
+                        if not tag.get('vocabulary_id')]
+    dictized["extras"] = dict((extra["key"], h.json.loads(extra["value"]))
                               for extra in dictized["extras"])
+    dictized['license'] = pkg.license.title if pkg.license else None
+    dictized['ratings_average'] = pkg.get_average_rating()
+    dictized['ratings_count'] = len(pkg.ratings)
     dictized['notes_rendered'] = ckan.misc.MarkdownFormat().to_html(pkg.notes)
 
-    resources = dictized["resources"] 
-   
-    for resource in resources:
+    site_url = config.get('ckan.site_url', None)
+    if site_url:
+        dictized['ckan_url'] = '%s/dataset/%s' % (site_url, pkg.name)
+
+    for resource in dictized["resources"]:
         resource_dict_to_api(resource, pkg.id, context)
 
-    if pkg.resources:
-        dictized['download_url'] = pkg.resources[0].url
-            
-    dictized['license'] = pkg.license.title if pkg.license else None
+    def make_api_1(package_id):
+        return pkg.get(package_id).name
 
-    dictized['ratings_average'] = pkg.get_average_rating()
-    dictized['ratings_count'] = len(pkg.ratings)
-    site_url = config.get('ckan.site_url', None)
-    if site_url:
-        dictized['ckan_url'] = '%s/dataset/%s' % (site_url, pkg.name)
-    metadata_modified = pkg.metadata_modified
-    dictized['metadata_modified'] = metadata_modified.isoformat() \
-        if metadata_modified else None
-    metadata_created = pkg.metadata_created
-    dictized['metadata_created'] = metadata_created.isoformat() \
-        if metadata_created else None
+    def make_api_2(package_id):
+        return package_id
+
+    if api_version == 1:
+        api_fn = make_api_1
+        dictized["groups"] = [group["name"] for group in dictized["groups"]]
+        # FIXME why is this just for version 1?
+        if pkg.resources:
+            dictized['download_url'] = pkg.resources[0].url
+    else:
+        api_fn = make_api_2
+        dictized["groups"] = [group["id"] for group in dictized["groups"]]
+
+    subjects = dictized.pop("relationships_as_subject")
+    objects = dictized.pop("relationships_as_object")
 
-    subjects = dictized.pop("relationships_as_subject") 
-    objects = dictized.pop("relationships_as_object") 
-    
     relationships = []
-    for relationship in objects:
+    for rel in objects:
         model = context['model']
         swap_types = model.PackageRelationship.forward_to_reverse_type
-        type = swap_types(relationship['type'])
-        relationships.append({'subject': pkg.get(relationship['object_package_id']).name,
+        type = swap_types(rel['type'])
+        relationships.append({'subject': api_fn(rel['object_package_id']),
                               'type': type,
-                              'object': pkg.get(relationship['subject_package_id']).name,
-                              'comment': relationship["comment"]})
-    for relationship in subjects:
-        model = context['model']
-        relationships.append({'subject': pkg.get(relationship['subject_package_id']).name,
-                              'type': relationship['type'],
-                              'object': pkg.get(relationship['object_package_id']).name,
-                              'comment': relationship["comment"]})
-        
-        
-    dictized['relationships'] = relationships 
+                              'object': api_fn(rel['subject_package_id']),
+                              'comment': rel["comment"]})
+    for rel in subjects:
+        relationships.append({'subject': api_fn(rel['subject_package_id']),
+                              'type': rel['type'],
+                              'object': api_fn(rel['object_package_id']),
+                              'comment': rel["comment"]})
+
+    dictized['relationships'] = relationships
+
     return dictized
 
-def package_to_api2(pkg, context):
+def vocabulary_dictize(vocabulary, context):
+    vocabulary_dict = d.table_dictize(vocabulary, context)
+    assert not vocabulary_dict.has_key('tags')
+    vocabulary_dict['tags'] = [tag_dictize(tag, context) for tag
+            in vocabulary.tags]
+    return vocabulary_dict
 
-    dictized = package_dictize(pkg, context)
-
-    dictized["groups"] = [group["id"] for group in dictized["groups"]]
-    dictized.pop("revision_timestamp")
-    
-    dictized["tags"] = [tag["name"] for tag in dictized["tags"]]
-    dictized["extras"] = dict((extra["key"], json.loads(extra["value"])) 
-                              for extra in dictized["extras"])
-
-    resources = dictized["resources"] 
-   
-    for resource in resources:
-        resource_dict_to_api(resource,pkg.id, context)
-            
-    dictized['license'] = pkg.license.title if pkg.license else None
-
-    dictized['ratings_average'] = pkg.get_average_rating()
-    dictized['ratings_count'] = len(pkg.ratings)
-    site_url = config.get('ckan.site_url', None)
-    if site_url:
-        dictized['ckan_url'] = '%s/dataset/%s' % (site_url, pkg.name)
-    dictized['metadata_modified'] = pkg.metadata_modified.isoformat() \
-        if pkg.metadata_modified else None
-    dictized['metadata_created'] = pkg.metadata_created.isoformat() \
-        if pkg.metadata_created else None
-    dictized['notes_rendered'] = ckan.misc.MarkdownFormat().to_html(pkg.notes)
-
-    subjects = dictized.pop("relationships_as_subject") 
-    objects = dictized.pop("relationships_as_object") 
-    
-    relationships = []
-    for relationship in objects:
-        model = context['model']
-        swap_types = model.PackageRelationship.forward_to_reverse_type
-        type = swap_types(relationship['type'])
-        relationships.append({'subject': relationship['object_package_id'],
-                              'type': type,
-                              'object': relationship['subject_package_id'],
-                              'comment': relationship["comment"]})
-    for relationship in subjects:
-        model = context['model']
-        relationships.append({'subject': relationship['subject_package_id'],
-                              'type': relationship['type'],
-                              'object': relationship['object_package_id'],
-                              'comment': relationship["comment"]})
-        
-    dictized['relationships'] = relationships 
-    return dictized
+def vocabulary_list_dictize(vocabulary_list, context):
+    return [vocabulary_dictize(vocabulary, context)
+            for vocabulary in vocabulary_list]
 
 def activity_dictize(activity, context):
-    activity_dict = table_dictize(activity, context)
+    activity_dict = d.table_dictize(activity, context)
     return activity_dict
 
 def activity_list_dictize(activity_list, context):
-    activity_dicts = []
-    for activity in activity_list:
-        activity_dict = activity_dictize(activity, context)
-        activity_dicts.append(activity_dict)
-    return activity_dicts
+    return [activity_dictize(activity, context) for activity in activity_list]
 
 def activity_detail_dictize(activity_detail, context):
-    return table_dictize(activity_detail, context)
+    return d.table_dictize(activity_detail, context)
 
 def activity_detail_list_dictize(activity_detail_list, context):
-    activity_detail_dicts = []
-    for activity_detail in activity_detail_list:
-        activity_detail_dict = activity_detail_dictize(activity_detail,
-            context)
-        activity_detail_dicts.append(activity_detail_dict)
-    return activity_detail_dicts
+    return [activity_detail_dictize(activity_detail, context)
+            for activity_detail in activity_detail_list]
+
+
+def package_to_api1(pkg, context):
+    # DEPRICIATED set api_version in context and use package_to_api()
+    context['api_version'] = 1
+    return package_to_api(pkg, context)
+
+def package_to_api2(pkg, context):
+    # DEPRICIATED set api_version in context and use package_to_api()
+    context['api_version'] = 2
+    return package_to_api(pkg, context)
+
+def group_to_api1(group, context):
+    # DEPRICIATED set api_version in context and use group_to_api()
+    context['api_version'] = 1
+    return group_to_api(group, context)
+
+def group_to_api2(group, context):
+    # DEPRICIATED set api_version in context and use group_to_api()
+    context['api_version'] = 2
+    return group_to_api(group, context)
+
+def tag_to_api1(tag, context):
+    # DEPRICIATED set api_version in context and use tag_to_api()
+    context['api_version'] = 1
+    return tag_to_api(tag, context)
+
+def tag_to_api2(tag, context):
+    # DEPRICIATED set api_version in context and use tag_to_api()
+    context['api_version'] = 2
+    return tag_to_api(tag, context)
