@@ -16,7 +16,7 @@ VALID_SOLR_PARAMETERS = set([
     'extras' # Not used by Solr, but useful for extensions
 ])
 
-# for (solr) package searches, this specifies the fields that are searched 
+# for (solr) package searches, this specifies the fields that are searched
 # and their relative weighting
 QUERY_FIELDS = "name^4 title^4 tags^2 groups^2 text"
 
@@ -65,30 +65,30 @@ def convert_legacy_parameters_to_solr(legacy_params):
         log.debug('Converted legacy search params from %r to %r',
                  legacy_params, solr_params)
     return solr_params
-    
+
 
 class QueryOptions(dict):
     """
-    Options specify aspects of the search query which are only tangentially related 
+    Options specify aspects of the search query which are only tangentially related
     to the query terms (such as limits, etc.).
     NB This is used only by legacy package search and current resource & tag search.
        Modern SOLR package search leaves this to SOLR syntax.
     """
-    
+
     BOOLEAN_OPTIONS = ['all_fields']
     INTEGER_OPTIONS = ['offset', 'limit']
     UNSUPPORTED_OPTIONS = ['filter_by_downloadable', 'filter_by_openness']
 
     def __init__(self, **kwargs):
         from ckan.lib.search import DEFAULT_OPTIONS
-        
+
         # set values according to the defaults
         for option_name, default_value in DEFAULT_OPTIONS.items():
             if not option_name in self:
                 self[option_name] = default_value
-        
+
         super(QueryOptions, self).__init__(**kwargs)
-    
+
     def validate(self):
         for key, value in self.items():
             if key in self.BOOLEAN_OPTIONS:
@@ -102,31 +102,31 @@ class QueryOptions(dict):
                 except ValueError:
                     raise SearchQueryError('Value for search option %r must be an integer but received %r' % (key, value))
             elif key in self.UNSUPPORTED_OPTIONS:
-                    raise SearchQueryError('Search option %r is not supported' % key)                
-            self[key] = value    
-    
+                    raise SearchQueryError('Search option %r is not supported' % key)
+            self[key] = value
+
     def __getattr__(self, name):
         return self.get(name)
-        
+
     def __setattr__(self, name, value):
         self[name] = value
 
 
 class SearchQuery(object):
     """
-    A query is ... when you ask the search engine things. SearchQuery is intended 
+    A query is ... when you ask the search engine things. SearchQuery is intended
     to be used for only one query, i.e. it sets state. Definitely not thread-safe.
     """
-    
+
     def __init__(self):
         self.results = []
         self.count = 0
-    
+
     @property
     def open_licenses(self):
         # this isn't exactly the very best place to put these, but they stay
-        # there persistently. 
-        # TODO: figure out if they change during run-time. 
+        # there persistently.
+        # TODO: figure out if they change during run-time.
         global _open_licenses
         if not isinstance(_open_licenses, list):
             _open_licenses = []
@@ -134,16 +134,16 @@ class SearchQuery(object):
                 if license and license.isopen():
                     _open_licenses.append(license.id)
         return _open_licenses
-    
+
     def get_all_entity_ids(self, max_results=1000):
         """
         Return a list of the IDs of all indexed packages.
         """
         return []
-    
+
     def run(self, query=None, terms=[], fields={}, facet_by=[], options=None, **kwargs):
         raise SearchError("SearchQuery.run() not implemented!")
-        
+
     # convenience, allows to query(..)
     __call__ = run
 
@@ -152,13 +152,13 @@ class TagSearchQuery(SearchQuery):
     """Search for tags."""
     def run(self, query=[], fields={}, options=None, **kwargs):
         if options is None:
-            options = QueryOptions(**kwargs) 
+            options = QueryOptions(**kwargs)
         else:
             options.update(kwargs)
 
         context = {'model': model, 'session': model.Session}
         data_dict = {
-            'query': query, 
+            'query': query,
             'fields': fields,
             'offset': options.get('offset'),
             'limit': options.get('limit')
@@ -171,8 +171,8 @@ class TagSearchQuery(SearchQuery):
             if options.all_fields:
                 results['results'] = [r.as_dict() for r in results['results']]
             else:
-                results['results'] = [r.name for r in results['results']]
-        
+                results['results'] = [r['name'] for r in results['results']]
+
         self.count = results['count']
         self.results = results['results']
         return results
@@ -182,7 +182,7 @@ class ResourceSearchQuery(SearchQuery):
     """Search for resources."""
     def run(self, fields={}, options=None, **kwargs):
         if options is None:
-            options = QueryOptions(**kwargs) 
+            options = QueryOptions(**kwargs)
         else:
             options.update(kwargs)
 
@@ -225,13 +225,41 @@ class PackageSearchQuery(SearchQuery):
 
         return [r.get('id') for r in data.results]
 
+    def get_index(self,reference):
+        query = {
+            'rows': 1,
+            'q': 'name:%s OR id:%s' % (reference,reference),
+            'wt': 'json',
+            'fq': 'site_id:"%s"' % config.get('ckan.site_id')}
+
+        conn = make_connection()
+        log.debug('Package query: %r' % query)
+        try:
+            solr_response = conn.raw_query(**query)
+        except SolrException, e:
+            raise SearchError('SOLR returned an error running query: %r Error: %r' %
+                              (query, e.reason))
+        try:
+            data = json.loads(solr_response)
+
+            if data['response']['numFound'] == 0:
+             raise SearchError('Dataset not found in the search index: %s' % reference)
+            else:
+                return data['response']['docs'][0]
+        except Exception, e:
+            log.exception(e)
+            raise SearchError(e)
+        finally:
+            conn.close()
+
+
     def run(self, query):
         '''
         Performs a dataset search using the given query.
 
         @param query - dictionary with keys like: q, fq, sort, rows, facet
         @return - dictionary with keys results and count
-        
+
         May raise SearchQueryError or SearchError.
         '''
         from solr import SolrException
@@ -258,7 +286,7 @@ class PackageSearchQuery(SearchQuery):
 
         # order by score if no 'sort' term given
         order_by = query.get('sort')
-        if order_by == 'rank' or order_by is None: 
+        if order_by == 'rank' or order_by is None:
             query['sort'] = 'score desc, name asc'
 
         # show only results from this CKAN instance
@@ -266,7 +294,7 @@ class PackageSearchQuery(SearchQuery):
         if not '+site_id:' in fq:
             fq += ' +site_id:"%s"' % config.get('ckan.site_id')
 
-        # filter for package status       
+        # filter for package status
         if not '+state:' in fq:
             fq += " +state:active"
         query['fq'] = fq
@@ -278,17 +306,16 @@ class PackageSearchQuery(SearchQuery):
 
         # return the package ID and search scores
         query['fl'] = query.get('fl', 'name')
-        
+
         # return results as json encoded string
         query['wt'] = query.get('wt', 'json')
 
-        # query field weighting: disabled for now as solr 3.* is required for 
-        # the 'edismax' query parser, our current Ubuntu version only has
-        # packages for 1.4
-        #
-        # query['defType'] = 'edismax'
-        # query['tie'] = '0.5'
-        # query['qf'] = query.get('qf', QUERY_FIELDS)
+        # If the query has a colon in it then consider it a fielded search and do use dismax.
+        if ':' not in query['q']:
+            query['defType'] = 'dismax'
+            query['tie'] = '0.1'
+            query['mm'] = '1'
+            query['qf'] = query.get('qf', QUERY_FIELDS)
 
         conn = make_connection()
         log.debug('Package query: %r' % query)
@@ -329,5 +356,5 @@ class PackageSearchQuery(SearchQuery):
             raise SearchError(e)
         finally:
             conn.close()
-        
+
         return {'results': self.results, 'count': self.count}
