@@ -1,5 +1,6 @@
 import uuid
 import logging
+import json
 
 from pylons import config
 from pylons.i18n import _
@@ -221,6 +222,7 @@ def group_list(context, data_dict):
     model = context['model']
     user = context['user']
     api = context.get('api_version')
+    groups = data_dict.get('groups')
     ref_group_by = 'id' if api == 2 else 'name';
     order_by = data_dict.get('order_by', 'name')
     if order_by not in set(('name', 'packages')):
@@ -232,6 +234,8 @@ def group_list(context, data_dict):
     query = model.Session.query(model.Group).join(model.GroupRevision)
     query = query.filter(model.GroupRevision.state=='active')
     query = query.filter(model.GroupRevision.current==True)
+    if groups:
+        query = query.filter(model.GroupRevision.name.in_(groups))
 
     if order_by == 'name':
         sort_by, reverse = 'name', False
@@ -321,7 +325,7 @@ def tag_list(context, data_dict):
     check_access('tag_list', context, data_dict)
 
     if query:
-        tags = _tag_search(context, data_dict)
+        tags, count = _tag_search(context, data_dict)
     else:
         tags = model.Tag.all(vocab_id_or_name)
 
@@ -802,13 +806,24 @@ def package_search(context, data_dict):
     results = []
     if not abort:
         # return a list of package ids
-        data_dict['fl'] = 'id'
+        data_dict['fl'] = 'id data_dict'
+
+
+        # If this query hasn't come from a controller that has set this flag
+        # then we should remove any mention of capacity from the fq and
+        # instead set it to only retrieve public datasets
+        fq = data_dict.get('fq','')
+        if not context.get('ignore_capacity_check',False):
+            fq = ' '.join(p for p in fq.split(' ')
+                            if not 'capacity:' in p)
+            data_dict['fq'] = fq + ' capacity:"public"'
 
         query = search.query_for(model.Package)
         query.run(data_dict)
 
         for package in query.results:
             # get the package object
+            package, package_dict = package['id'], package.get('data_dict')
             pkg_query = session.query(model.PackageRevision)\
                 .filter(model.PackageRevision.id == package)\
                 .filter(and_(
@@ -822,9 +837,16 @@ def package_search(context, data_dict):
             if not pkg:
                 log.warning('package %s in index but not in database' % package)
                 continue
-
-            result_dict = model_dictize.package_dictize(pkg,context)
-            results.append(result_dict)
+            ## use data in search index if there
+            if package_dict:
+                ## the package_dict still needs translating when being viewed
+                package_dict = json.loads(package_dict)
+                if context.get('for_view'):
+                    for item in plugins.PluginImplementations( plugins.IPackageController):
+                        package_dict = item.before_view(package_dict)
+                results.append(package_dict)
+            else:
+                results.append(model_dictize.package_dictize(pkg,context))
 
         count = query.count
         facets = query.facets
@@ -926,7 +948,8 @@ def resource_search(context, data_dict):
     return {'count': count, 'results': results}
 
 def _tag_search(context, data_dict):
-    '''Return a list of tag objects that contain the given string.
+    '''Return a list of tag objects that contain the given string and
+    the full count (for paging).
 
     The query string should be provided in the data_dict with key 'query' or
     'q'.
@@ -935,6 +958,7 @@ def _tag_search(context, data_dict):
     searched. If a 'vocabulary_id' is provided in the data_dict then tags
     belonging to the given vocabulary (id or name) will be searched instead.
 
+    Use 'offset' and 'limit' parameters to page through results.
     '''
     model = context['model']
 
@@ -968,15 +992,16 @@ def _tag_search(context, data_dict):
             terms.append(value)
 
     if not len(terms):
-        return []
+        return [], 0
 
     for term in terms:
         escaped_term = misc.escape_sql_like_special_characters(term, escape='\\')
         q = q.filter(model.Tag.name.ilike('%' + escaped_term + '%'))
 
+    count = q.count()
     q = q.offset(offset)
     q = q.limit(limit)
-    return q.all()
+    return q.all(), count
 
 def tag_search(context, data_dict):
     '''Return a list of tag dictionaries that contain the given string.
@@ -992,8 +1017,8 @@ def tag_search(context, data_dict):
     and 'results' (the list of tag dicts).
 
     '''
-    tags = _tag_search(context, data_dict)
-    return {'count': len(tags),
+    tags, count = _tag_search(context, data_dict)
+    return {'count': count,
             'results': [table_dictize(tag, context) for tag in tags]}
 
 def tag_autocomplete(context, data_dict):
@@ -1008,7 +1033,7 @@ def tag_autocomplete(context, data_dict):
 
     '''
     check_access('tag_autocomplete', context, data_dict)
-    matching_tags = _tag_search(context, data_dict)
+    matching_tags, count = _tag_search(context, data_dict)
     if matching_tags:
         return [tag.name for tag in matching_tags]
     else:
