@@ -1,95 +1,28 @@
 import logging
-import re
 import datetime
 
-from ckan.plugins import PluginImplementations, IGroupController, IPackageController
-from ckan.logic import NotFound, ValidationError, ParameterError
-from ckan.logic import check_access
-
-from ckan.lib.base import _
+from pylons.i18n import _
 from vdm.sqlalchemy.base import SQLAlchemySession
+
+import ckan.plugins as plugins
+import ckan.logic as logic
+import ckan.logic.schema
 import ckan.lib.dictization
-from ckan.lib.dictization.model_dictize import (package_dictize,
-                                                package_to_api1,
-                                                package_to_api2,
-                                                resource_dictize,
-                                                task_status_dictize,
-                                                group_dictize,
-                                                group_to_api1,
-                                                group_to_api2,
-                                                user_dictize)
-from ckan.lib.dictization.model_save import (group_api_to_dict,
-                                             package_api_to_dict,
-                                             group_dict_save,
-                                             user_dict_save,
-                                             task_status_dict_save,
-                                             package_dict_save,
-                                             resource_dict_save)
-from ckan.logic.schema import (default_update_group_schema,
-                               default_update_package_schema,
-                               default_update_user_schema,
-                               default_update_resource_schema,
-                               default_update_relationship_schema,
-                               default_task_status_schema)
-from ckan.lib.navl.dictization_functions import validate
-from ckan.logic.action import rename_keys, get_domain_object
-from ckan.logic.action.get import roles_show
+import ckan.lib.dictization.model_dictize as model_dictize
+import ckan.lib.dictization.model_save as model_save
+import ckan.lib.navl.dictization_functions
+import ckan.lib.navl.validators as validators
+import ckan.lib.plugins as lib_plugins
 
 log = logging.getLogger(__name__)
 
-def prettify(field_name):
-    field_name = re.sub('(?<!\w)[Uu]rl(?!\w)', 'URL', field_name.replace('_', ' ').capitalize())
-    return _(field_name.replace('_', ' '))
-
-def package_error_summary(error_dict):
-
-    error_summary = {}
-    for key, error in error_dict.iteritems():
-        if key == 'resources':
-            error_summary[_('Resources')] = _('Package resource(s) invalid')
-        elif key == 'extras':
-            error_summary[_('Extras')] = _('Missing Value')
-        elif key == 'extras_validation':
-            error_summary[_('Extras')] = error[0]
-        else:
-            error_summary[_(prettify(key))] = error[0]
-    return error_summary
-
-def resource_error_summary(error_dict):
-
-    error_summary = {}
-    for key, error in error_dict.iteritems():
-        if key == 'extras':
-            error_summary[_('Extras')] = _('Missing Value')
-        elif key == 'extras_validation':
-            error_summary[_('Extras')] = error[0]
-        else:
-            error_summary[_(prettify(key))] = error[0]
-    return error_summary
-
-def group_error_summary(error_dict):
-
-    error_summary = {}
-    for key, error in error_dict.iteritems():
-        if key == 'extras':
-            error_summary[_('Extras')] = _('Missing Value')
-        elif key == 'extras_validation':
-            error_summary[_('Extras')] = error[0]
-        else:
-            error_summary[_(prettify(key))] = error[0]
-    return error_summary
-
-def task_status_error_summary(error_dict):
-    error_summary = {}
-    for key, error in error_dict.iteritems():
-        error_summary[_(prettify(key))] = error[0]
-    return error_summary
-
-def relationship_error_summary(error_dict):
-    error_summary = {}
-    for key, error in error_dict.iteritems():
-        error_summary[_(prettify(key))] = error[0]
-    return error_summary
+# define some shortcuts
+validate = ckan.lib.navl.dictization_functions.validate
+error_summary = logic.action.error_summary
+get_action = logic.get_action
+check_access = logic.check_access
+NotFound = logic.NotFound
+ValidationError = logic.ValidationError
 
 def _make_latest_rev_active(context, q):
 
@@ -110,7 +43,7 @@ def _make_latest_rev_active(context, q):
         latest_rev.state = 'active'
 
     session.add(latest_rev)
-        
+
     ##this is just a way to get the latest revision that changed
     ##in order to timestamp
     old_latest = context.get('latest_revision_date')
@@ -159,18 +92,46 @@ def make_latest_pending_package_active(context, data_dict):
     revision = q.first()
     revision.approved_timestamp = datetime.datetime.now()
     session.add(revision)
-    
+
     if not context.get('defer_commit'):
-        session.commit()        
-    session.remove()        
+        session.commit()
+    session.remove()
+
+
+def related_update(context, data_dict):
+    model = context['model']
+    user = context['user']
+    id = data_dict["id"]
+
+    schema = context.get('schema') or ckan.logic.schema.default_related_schema()
+    model.Session.remove()
+
+    related = model.Related.get(id)
+    context["related"] = related
+
+    if not related:
+        logging.error('Could not find related ' + id)
+        raise NotFound(_('Related was not found.'))
+
+    check_access('related_update', context, data_dict)
+    data, errors = validate(data_dict, schema, context)
+
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors, error_summary(errors))
+
+    related = model_save.related_dict_save(data, context)
+    if not context.get('defer_commit'):
+        model.repo.commit()
+    return model_dictize.related_dictize(related, context)
+
 
 
 def resource_update(context, data_dict):
     model = context['model']
-    session = context['session']
     user = context['user']
     id = data_dict["id"]
-    schema = context.get('schema') or default_update_resource_schema()
+    schema = context.get('schema') or ckan.logic.schema.default_update_resource_schema()
     model.Session.remove()
 
     resource = model.Resource.get(id)
@@ -186,7 +147,7 @@ def resource_update(context, data_dict):
 
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, resource_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -195,36 +156,51 @@ def resource_update(context, data_dict):
     else:
         rev.message = _(u'REST API: Update object %s') % data.get("name", "")
 
-    resource = resource_dict_save(data, context)
+    resource = model_save.resource_dict_save(data, context)
     if not context.get('defer_commit'):
-        model.repo.commit()        
-    return resource_dictize(resource, context)
+        model.repo.commit()
+    return model_dictize.resource_dictize(resource, context)
+
 
 
 def package_update(context, data_dict):
+
     model = context['model']
     user = context['user']
-    
-    id = data_dict["id"]
-    schema = context.get('schema') or default_update_package_schema()
+    name_or_id = data_dict.get("id") or data_dict['name']
     model.Session.remove()
     model.Session()._context = context
 
-    pkg = model.Package.get(id)
-    context["package"] = pkg
-
+    pkg = model.Package.get(name_or_id)
     if pkg is None:
         raise NotFound(_('Package was not found.'))
+    context["package"] = pkg
     data_dict["id"] = pkg.id
 
     check_access('package_update', context, data_dict)
 
+    # get the schema
+    package_plugin = lib_plugins.lookup_package_plugin(pkg.type)
+    try:
+        schema = package_plugin.form_to_db_schema_options({'type':'update',
+                                               'api':'api_version' in context,
+                                               'context': context})
+    except AttributeError:
+        schema = package_plugin.form_to_db_schema()
+
+    if 'api_version' not in context:
+        # old plugins do not support passing the schema so we need
+        # to ensure they still work
+        try:
+            package_plugin.check_data_dict(data_dict, schema)
+        except TypeError:
+            package_plugin.check_data_dict(data_dict)
+
     data, errors = validate(data_dict, schema, context)
-    
 
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, package_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
     rev = model.repo.new_revision()
     rev.author = user
@@ -233,20 +209,19 @@ def package_update(context, data_dict):
     else:
         rev.message = _(u'REST API: Update object %s') % data.get("name")
 
-    pkg = package_dict_save(data, context)
+    pkg = model_save.package_dict_save(data, context)
 
-    for item in PluginImplementations(IPackageController):
+    for item in plugins.PluginImplementations(plugins.IPackageController):
         item.edit(pkg)
     if not context.get('defer_commit'):
-        model.repo.commit()        
-    return package_dictize(pkg, context)
+        model.repo.commit()
+    return get_action('package_show')(context, data_dict)
 
 def package_update_validate(context, data_dict):
     model = context['model']
     user = context['user']
-    
+
     id = data_dict["id"]
-    schema = context.get('schema') or default_update_package_schema()
     model.Session.remove()
     model.Session()._context = context
 
@@ -257,6 +232,15 @@ def package_update_validate(context, data_dict):
         raise NotFound(_('Package was not found.'))
     data_dict["id"] = pkg.id
 
+    # get the schema
+    package_plugin = lib_plugins.lookup_package_plugin(pkg.type)
+    try:
+        schema = package_plugin.form_to_db_schema_options({'type':'update',
+                                               'api':'api_version' in context,
+                                               'context': context})
+    except AttributeError:
+        schema = package_plugin.form_to_db_schema()
+
     check_access('package_update', context, data_dict)
 
     data, errors = validate(data_dict, schema, context)
@@ -264,19 +248,19 @@ def package_update_validate(context, data_dict):
 
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, package_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
     return data
 
 
 def _update_package_relationship(relationship, comment, context):
     model = context['model']
-    api = context.get('api_version') or '1'
-    ref_package_by = 'id' if api == '2' else 'name'
+    api = context.get('api_version')
+    ref_package_by = 'id' if api == 2 else 'name'
     is_changed = relationship.comment != comment
     if is_changed:
         rev = model.repo.new_revision()
         rev.author = context["user"]
-        rev.message = (_(u'REST API: Update package relationship: %s %s %s') % 
+        rev.message = (_(u'REST API: Update package relationship: %s %s %s') %
             (relationship.subject, relationship.type, relationship.object))
         relationship.comment = comment
         if not context.get('defer_commit'):
@@ -289,13 +273,11 @@ def package_relationship_update(context, data_dict):
 
     model = context['model']
     user = context['user']
-    schema = context.get('schema') or default_update_relationship_schema()
-    api = context.get('api_version') or '1'
+    schema = context.get('schema') or ckan.logic.schema.default_update_relationship_schema()
 
     id = data_dict['subject']
     id2 = data_dict['object']
     rel = data_dict['type']
-    ref_package_by = 'id' if api == '2' else 'name'
 
     pkg1 = model.Package.get(id)
     pkg2 = model.Package.get(id2)
@@ -308,7 +290,7 @@ def package_relationship_update(context, data_dict):
 
     if errors:
         model.Session.rollback()
-        raise ValidationError(errors, relationship_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
     check_access('package_relationship_update', context, data_dict)
 
@@ -324,32 +306,54 @@ def group_update(context, data_dict):
     model = context['model']
     user = context['user']
     session = context['session']
-    schema = context.get('schema') or default_update_group_schema()
     id = data_dict['id']
+    parent = context.get('parent', None)
 
     group = model.Group.get(id)
     context["group"] = group
     if group is None:
         raise NotFound('Group was not found.')
 
+    # get the schema
+    group_plugin = lib_plugins.lookup_group_plugin(group.type)
+    try:
+        schema = group_plugin.form_to_db_schema_options({'type':'update',
+                                               'api':'api_version' in context,
+                                               'context': context})
+    except AttributeError:
+        schema = group_plugin.form_to_db_schema()
+
     check_access('group_update', context, data_dict)
 
     data, errors = validate(data_dict, schema, context)
     if errors:
         session.rollback()
-        raise ValidationError(errors, group_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
     rev = model.repo.new_revision()
     rev.author = user
-    
+
     if 'message' in context:
         rev.message = context['message']
     else:
         rev.message = _(u'REST API: Update object %s') % data.get("name")
 
-    group = group_dict_save(data, context)
+    group = model_save.group_dict_save(data, context)
 
-    for item in PluginImplementations(IGroupController):
+    if parent:
+        parent_group = model.Group.get( parent )
+        if parent_group and not parent_group in group.get_groups(group.type):
+            # Delete all of this groups memberships
+            current = session.query(model.Member).\
+               filter(model.Member.table_id == group.id).\
+               filter(model.Member.table_name == "group").all()
+            for c in current:
+                session.delete(c)
+            member = model.Member(group=parent_group, table_id=group.id, table_name='group')
+            session.add(member)
+
+
+    for item in plugins.PluginImplementations(plugins.IGroupController):
         item.edit(group)
 
     activity_dict = {
@@ -375,22 +379,21 @@ def group_update(context, data_dict):
         activity_dict['data'] = {
                 'group': ckan.lib.dictization.table_dictize(group, context)
                 }
-        from ckan.logic.action.create import activity_create
         activity_create_context = {
             'model': model,
             'user': user,
             'defer_commit':True,
             'session': session
         }
-        activity_create(activity_create_context, activity_dict,
+        get_action('activity_create')(activity_create_context, activity_dict,
                 ignore_auth=True)
         # TODO: Also create an activity detail recording what exactly changed
         # in the group.
 
     if not context.get('defer_commit'):
-        model.repo.commit()        
+        model.repo.commit()
 
-    return group_dictize(group, context)
+    return model_dictize.group_dictize(group, context)
 
 def user_update(context, data_dict):
     '''Updates the user\'s details'''
@@ -398,7 +401,7 @@ def user_update(context, data_dict):
     model = context['model']
     user = context['user']
     session = context['session']
-    schema = context.get('schema') or default_update_user_schema() 
+    schema = context.get('schema') or ckan.logic.schema.default_update_user_schema()
     id = data_dict['id']
 
     user_obj = model.User.get(id)
@@ -411,9 +414,9 @@ def user_update(context, data_dict):
     data, errors = validate(data_dict, schema, context)
     if errors:
         session.rollback()
-        raise ValidationError(errors, group_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
-    user = user_dict_save(data, context)
+    user = model_save.user_dict_save(data, context)
 
     activity_dict = {
             'user_id': user.id,
@@ -426,14 +429,13 @@ def user_update(context, data_dict):
         'defer_commit':True,
         'session': session
     }
-    from ckan.logic.action.create import activity_create
-    activity_create(activity_create_context, activity_dict, ignore_auth=True)
+    get_action('activity_create')(activity_create_context, activity_dict, ignore_auth=True)
     # TODO: Also create an activity detail recording what exactly changed in
     # the user.
 
     if not context.get('defer_commit'):
-        model.repo.commit()        
-    return user_dictize(user, context)
+        model.repo.commit()
+    return model_dictize.user_dictize(user, context)
 
 def task_status_update(context, data_dict):
     model = context['model']
@@ -442,7 +444,7 @@ def task_status_update(context, data_dict):
 
     user = context['user']
     id = data_dict.get("id")
-    schema = context.get('schema') or default_task_status_schema()
+    schema = context.get('schema') or ckan.logic.schema.default_task_status_schema()
 
     if id:
         task_status = model.TaskStatus.get(id)
@@ -450,20 +452,20 @@ def task_status_update(context, data_dict):
 
         if task_status is None:
             raise NotFound(_('TaskStatus was not found.'))
-    
+
     check_access('task_status_update', context, data_dict)
 
     data, errors = validate(data_dict, schema, context)
 
     if errors:
         session.rollback()
-        raise ValidationError(errors, task_status_error_summary(errors))
+        raise ValidationError(errors, error_summary(errors))
 
-    task_status = task_status_dict_save(data, context)
+    task_status = model_save.task_status_dict_save(data, context)
 
     session.commit()
     session.close()
-    return task_status_dictize(task_status, context)
+    return model_dictize.task_status_dictize(task_status, context)
 
 def task_status_update_many(context, data_dict):
     results = []
@@ -471,13 +473,67 @@ def task_status_update_many(context, data_dict):
     deferred = context.get('defer_commit')
     context['defer_commit'] = True
     for data in data_dict['data']:
-        results.append(task_status_update(context, data))
+        results.append(get_action('task_status_update')(context, data))
     if not deferred:
         context.pop('defer_commit')
     if not context.get('defer_commit'):
         model.Session.commit()
     return {'results': results}
 
+def term_translation_update(context, data_dict):
+    model = context['model']
+
+    check_access('term_translation_update', context, data_dict)
+
+    schema = {'term': [validators.not_empty, unicode],
+              'term_translation': [validators.not_empty, unicode],
+              'lang_code': [validators.not_empty, unicode]}
+
+    data, errors = validate(data_dict, schema, context)
+
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors)
+
+    trans_table = model.term_translation_table
+
+    update = trans_table.update()
+    update = update.where(trans_table.c.term == data['term'])
+    update = update.where(trans_table.c.lang_code == data['lang_code'])
+    update = update.values(term_translation = data['term_translation'])
+
+    conn = model.Session.connection()
+    result = conn.execute(update)
+
+    # insert if not updated
+    if not result.rowcount:
+        conn.execute(trans_table.insert().values(**data))
+
+    if not context.get('defer_commit'):
+        model.Session.commit()
+
+    return data
+
+def term_translation_update_many(context, data_dict):
+    model = context['model']
+
+
+    if not data_dict.get('data') and isinstance(data_dict, list):
+        raise ValidationError(
+            {'error':
+             'term_translation_update_many needs to have a list of dicts in field data'}
+        )
+
+    context['defer_commit'] = True
+
+    for num, row in enumerate(data_dict['data']):
+        term_translation_update(context, row)
+
+    model.Session.commit()
+
+    return {'success': '%s rows updated' % (num + 1)}
+
+
 ## Modifications for rest api
 
 def package_update_rest(context, data_dict):
@@ -485,13 +541,11 @@ def package_update_rest(context, data_dict):
     model = context['model']
     id = data_dict.get("id")
     request_id = context['id']
-    api = context.get('api_version') or '1'
     pkg = model.Package.get(request_id)
 
     if not pkg:
         raise NotFound
 
-
     if id and id != pkg.id:
         pkg_from_data = model.Package.get(id)
         if pkg_from_data != pkg:
@@ -500,20 +554,16 @@ def package_update_rest(context, data_dict):
             raise ValidationError(error_dict)
 
     context["package"] = pkg
-    context["allow_partial_update"] = True
-    dictized_package = package_api_to_dict(data_dict, context)
+    context["allow_partial_update"] = False
+    dictized_package = model_save.package_api_to_dict(data_dict, context)
 
     check_access('package_update_rest', context, dictized_package)
 
-    dictized_after = package_update(context, dictized_package)
-
+    dictized_after = get_action('package_update')(context, dictized_package)
 
     pkg = context['package']
 
-    if api == '1':
-        package_dict = package_to_api1(pkg, context)
-    else:
-        package_dict = package_to_api2(pkg, context)
+    package_dict = model_dictize.package_to_api(pkg, context)
 
     return package_dict
 
@@ -521,26 +571,53 @@ def group_update_rest(context, data_dict):
 
     model = context['model']
     id = data_dict["id"]
-    api = context.get('api_version') or '1'
     group = model.Group.get(id)
     context["group"] = group
     context["allow_partial_update"] = True
-    dictized_group = group_api_to_dict(data_dict, context)
+    dictized_group = model_save.group_api_to_dict(data_dict, context)
 
     check_access('group_update_rest', context, dictized_group)
 
-    dictized_after = group_update(context, dictized_group)
+    dictized_after = get_action('group_update')(context, dictized_group)
 
     group = context['group']
 
-
-    if api == '1':
-        group_dict = group_to_api1(group, context)
-    else:
-        group_dict = group_to_api2(group, context)
+    group_dict = model_dictize.group_to_api(group, context)
 
     return group_dict
 
+def vocabulary_update(context, data_dict):
+    model = context['model']
+
+    vocab_id = data_dict.get('id')
+    if not vocab_id:
+        raise ValidationError({'id': _('id not in data')})
+
+    vocab = model.vocabulary.Vocabulary.get(vocab_id)
+    if vocab is None:
+        raise NotFound(_('Could not find vocabulary "%s"') % vocab_id)
+
+    data_dict['id'] = vocab.id
+    if data_dict.has_key('name'):
+        if data_dict['name'] == vocab.name:
+            del data_dict['name']
+
+    check_access('vocabulary_update', context, data_dict)
+
+    schema = context.get('schema') or ckan.logic.schema.default_update_vocabulary_schema()
+    data, errors = validate(data_dict, schema, context)
+
+    if errors:
+        model.Session.rollback()
+        raise ValidationError(errors)
+
+    updated_vocab = model_save.vocabulary_dict_update(data, context)
+
+    if not context.get('defer_commit'):
+        model.repo.commit()
+
+    return model_dictize.vocabulary_dictize(updated_vocab, context)
+
 def package_relationship_update_rest(context, data_dict):
 
     # rename keys
@@ -552,9 +629,9 @@ def package_relationship_update_rest(context, data_dict):
     # object and rel in the URI overwrite any values for these
     # in params. This is because you are not allowed to change
     # these values.
-    data_dict = rename_keys(data_dict, key_map, destructive=True)
+    data_dict = logic.action.rename_keys(data_dict, key_map, destructive=True)
 
-    relationship_dict = package_relationship_update(context, data_dict)
+    relationship_dict = get_action('package_relationship_update')(context, data_dict)
 
     return relationship_dict
 
@@ -568,10 +645,10 @@ def user_role_update(context, data_dict):
     new_user_ref = data_dict.get('user') # the user who is being given the new role
     new_authgroup_ref = data_dict.get('authorization_group') # the authgroup who is being given the new role
     if bool(new_user_ref) == bool(new_authgroup_ref):
-        raise ParameterError('You must provide either "user" or "authorization_group" parameter.')
+        raise logic.ParameterError('You must provide either "user" or "authorization_group" parameter.')
     domain_object_ref = data_dict['domain_object']
     if not isinstance(data_dict['roles'], (list, tuple)):
-        raise ParameterError('Parameter "%s" must be of type: "%s"' % ('role', 'list'))
+        raise logic.ParameterError('Parameter "%s" must be of type: "%s"' % ('role', 'list'))
     desired_roles = set(data_dict['roles'])
 
     if new_user_ref:
@@ -584,12 +661,12 @@ def user_role_update(context, data_dict):
     else:
         user_object = model.AuthorizationGroup.get(new_authgroup_ref)
         if not user_object:
-            raise NotFound('Cannot find authorization group %r' % new_authgroup_ref)        
+            raise NotFound('Cannot find authorization group %r' % new_authgroup_ref)
         data_dict['authorization_group'] = user_object.id
         add_user_to_role_func = model.add_authorization_group_to_role
         remove_user_from_role_func = model.remove_authorization_group_from_role
 
-    domain_object = get_domain_object(model, domain_object_ref)
+    domain_object = logic.action.get_domain_object(model, domain_object_ref)
     data_dict['id'] = domain_object.id
     if isinstance(domain_object, model.Package):
         check_access('package_edit_permissions', context, data_dict)
@@ -599,12 +676,12 @@ def user_role_update(context, data_dict):
         check_access('authorization_group_edit_permissions', context, data_dict)
     # Todo: 'system' object
     else:
-        raise ParameterError('Not possible to update roles for domain object type %s' % type(domain_object))
+        raise logic.ParameterError('Not possible to update roles for domain object type %s' % type(domain_object))
 
     # current_uors: in order to avoid either creating a role twice or
     # deleting one which is non-existent, we need to get the users\'
     # current roles (if any)
-    current_role_dicts = roles_show(context, data_dict)['roles']
+    current_role_dicts = get_action('roles_show')(context, data_dict)['roles']
     current_roles = set([role_dict['role'] for role_dict in current_role_dicts])
 
     # Whenever our desired state is different from our current state,
@@ -618,7 +695,7 @@ def user_role_update(context, data_dict):
     if not (current_roles == desired_roles):
         model.repo.commit_and_remove()
 
-    return roles_show(context, data_dict)
+    return get_action('roles_show')(context, data_dict)
 
 def user_role_bulk_update(context, data_dict):
     '''
@@ -641,5 +718,4 @@ def user_role_bulk_update(context, data_dict):
                              'roles': roles_by_user[user],
                              'domain_object': data_dict['domain_object']}
             user_role_update(context, uro_data_dict)
-    return roles_show(context, data_dict)
-
+    return get_action('roles_show')(context, data_dict)
