@@ -31,9 +31,13 @@ from ckan.logic.validators import (package_id_not_changed,
                                    isodate,
                                    int_validator,
                                    user_about_validator,
+                                   vocabulary_name_validator,
+                                   vocabulary_id_not_changed,
+                                   vocabulary_id_exists,
                                    user_id_exists,
                                    object_id_validator,
-                                   activity_type_exists)
+                                   activity_type_exists,
+                                   tag_not_in_vocabulary)
 from formencode.validators import OneOf
 import ckan.model
 
@@ -58,9 +62,11 @@ def default_resource_schema():
         'webstore_url': [ignore_missing, unicode],
         'cache_url': [ignore_missing, unicode],
         'size': [ignore_missing, int_validator],
+        'created': [ignore_missing, isodate],
         'last_modified': [ignore_missing, isodate],
         'cache_last_updated': [ignore_missing, isodate],
         'webstore_last_updated': [ignore_missing, isodate],
+        'tracking_summary': [ignore],
         '__extras': [ignore_missing, extras_unicode_convert, keep_extras],
     }
 
@@ -71,18 +77,31 @@ def default_update_resource_schema():
     return schema
 
 def default_tags_schema():
-
     schema = {
-        'name': [not_empty,
+        'name': [not_missing,
+                 not_empty,
                  unicode,
                  tag_length_validator,
                  tag_name_validator,
                 ],
+        'vocabulary_id': [ignore_missing, unicode, vocabulary_id_exists],
         'revision_timestamp': [ignore],
         'state': [ignore],
+        'display_name': [ignore],
     }
     return schema
 
+def default_create_tag_schema():
+    schema = default_tags_schema()
+    # When creating a tag via the create_tag() logic action function, a
+    # vocabulary_id _must_ be given (you cannot create free tags via this
+    # function).
+    schema['vocabulary_id'] = [not_missing, not_empty, unicode,
+            vocabulary_id_exists, tag_not_in_vocabulary]
+    # You're not allowed to specify your own ID when creating a tag.
+    schema['id'] = [empty]
+    return schema
+
 def default_package_schema():
 
     schema = {
@@ -99,6 +118,7 @@ def default_package_schema():
         'url': [ignore_missing, unicode],#, URL(add_http=False)],
         'version': [ignore_missing, unicode, package_version_validator],
         'state': [ignore_not_package_admin, ignore_missing],
+        'type': [ignore_missing, unicode],
         '__extras': [ignore],
         '__junk': [empty],
         'resources': default_resource_schema(),
@@ -109,6 +129,7 @@ def default_package_schema():
         'groups': {
             'id': [ignore_missing, unicode],
             'name': [ignore_missing, unicode],
+            'title': [ignore_missing, unicode],
             '__extras': [ignore],
         }
     }
@@ -137,8 +158,7 @@ def package_form_schema():
     schema['log_message'] = [ignore_missing, unicode, no_http]
     schema['groups'] = {
             'id': [ignore_missing, unicode],
-            '__extras': [empty],
-            'name': [ignore, unicode],
+            '__extras': [ignore],
     }
     schema['tag_string'] = [ignore_missing, tag_string_convert]
     schema['extras_validation'] = [duplicate_extras_key, ignore]
@@ -162,13 +182,32 @@ def default_group_schema():
         'name': [not_empty, unicode, name_validator, group_name_validator],
         'title': [ignore_missing, unicode],
         'description': [ignore_missing, unicode],
-        'type': [ignore_missing, unicode],        
+        'image_url': [ignore_missing, unicode],
+        'type': [ignore_missing, unicode],
         'state': [ignore_not_group_admin, ignore_missing],
         'created': [ignore],
+        'approval_status': [ignore_missing, unicode],
         'extras': default_extras_schema(),
         '__extras': [ignore],
         'packages': {
             "id": [not_empty, unicode, package_id_or_name_exists],
+            "title":[ignore_missing, unicode],
+            "name":[ignore_missing, unicode],
+            "__extras": [ignore]
+        },
+         'groups': {
+            "name": [not_empty, unicode],
+            "capacity": [ignore_missing],
+            "__extras": [ignore]
+        },
+        'users': {
+            "name": [not_empty, unicode],
+            "capacity": [ignore_missing],
+            "__extras": [ignore]
+        },
+        'groups': {
+            "name": [not_empty, unicode],
+            "capacity": [ignore_missing],
             "__extras": [ignore]
         }
     }
@@ -179,6 +218,12 @@ def group_form_schema():
     #schema['extras_validation'] = [duplicate_extras_key, ignore]
     schema['packages'] = {
         "name": [not_empty, unicode],
+        "title": [ignore_missing],
+        "__extras": [ignore]
+    }
+    schema['users'] = {
+        "name": [not_empty, unicode],
+        "capacity": [ignore_missing],
         "__extras": [ignore]
     }
     return schema
@@ -190,6 +235,20 @@ def default_update_group_schema():
     return schema
 
 
+def default_related_schema():
+    schema = {
+        'id': [ignore_missing, unicode],
+        'title': [not_empty, unicode],
+        'description': [ignore_missing, unicode],
+        'type': [not_empty, unicode],
+        'image_url': [ignore_missing, unicode],
+        'url': [ignore_missing, unicode],
+        'owner_id': [not_empty, unicode],
+        'created': [ignore],
+    }
+    return schema
+
+
 def default_extras_schema():
 
     schema = {
@@ -294,6 +353,25 @@ def default_task_status_schema():
     }
     return schema
 
+def default_vocabulary_schema():
+    schema = {
+        'id': [ignore_missing, unicode, vocabulary_id_exists],
+        'name': [not_empty, unicode, vocabulary_name_validator],
+        'tags': default_tags_schema(),
+    }
+    return schema
+
+def default_create_vocabulary_schema():
+    schema = default_vocabulary_schema()
+    schema['id'] = [empty]
+    return schema
+
+def default_update_vocabulary_schema():
+    schema = default_vocabulary_schema()
+    schema['id'] = [ignore_missing, vocabulary_id_not_changed]
+    schema['name'] = [ignore_missing, vocabulary_name_validator]
+    return schema
+
 def default_create_activity_schema():
     schema = {
         'id': [ignore],
