@@ -47,7 +47,8 @@ class Action(Enum):
     SITE_READ = u'read-site'
     USER_READ = u'read-user'
     USER_CREATE = u'create-user'
-
+    UPLOAD_ACTION = u'file-upload'
+    
 class Role(Enum):
     ADMIN = u'admin'
     EDITOR = u'editor'
@@ -67,12 +68,14 @@ default_role_actions = [
     (Role.EDITOR, Action.USER_READ),
     (Role.EDITOR, Action.SITE_READ),
     (Role.EDITOR, Action.READ),
+    (Role.EDITOR, Action.UPLOAD_ACTION),    
     (Role.ANON_EDITOR, Action.EDIT),
     (Role.ANON_EDITOR, Action.PACKAGE_CREATE),
     (Role.ANON_EDITOR, Action.USER_CREATE),
     (Role.ANON_EDITOR, Action.USER_READ),
     (Role.ANON_EDITOR, Action.SITE_READ),
     (Role.ANON_EDITOR, Action.READ),
+    (Role.ANON_EDITOR, Action.UPLOAD_ACTION),        
     (Role.READER, Action.USER_CREATE),
     (Role.READER, Action.USER_READ),
     (Role.READER, Action.SITE_READ),
@@ -94,7 +97,7 @@ user_object_role_table = Table('user_object_role', metadata,
            Column('id', UnicodeText, primary_key=True, default=make_uuid),
            Column('user_id', UnicodeText, ForeignKey('user.id'), nullable=True),
            Column('authorized_group_id', UnicodeText, ForeignKey('authorization_group.id'), nullable=True),
-           Column('context', UnicodeText, nullable=False),
+           Column('context', UnicodeText, nullable=False), # stores subtype
            Column('role', UnicodeText)
            )
 
@@ -136,12 +139,10 @@ class UserObjectRole(DomainObject):
             return '<%s user="%s" role="%s" context="%s">' % \
                 (self.__class__.__name__, self.user.name, self.role, self.context)
         elif self.authorized_group:
-            return '<%s group="%s" role="%s" context="%s">' % \
+            return '<%s authorized_group="%s" role="%s" context="%s">' % \
                 (self.__class__.__name__, self.authorized_group.name, self.role, self.context)
         else:
             assert False, "UserObjectRole is neither for an authzgroup or for a user" 
-            
-
 
     @classmethod
     def get_object_role_class(cls, domain_obj):
@@ -208,7 +209,7 @@ class UserObjectRole(DomainObject):
         commit, will add the role to the database twice. Since some other
         functions count the number of occurrences, that leaves a fairly obvious
         bug. But adding a commit here seems to break various tests.
-        So don't call this twice without committing, I guess...
+        So don\'t call this twice without committing, I guess...
         '''
         if cls.authorization_group_has_role(authorization_group, role, domain_obj):
             return
@@ -236,16 +237,49 @@ class UserObjectRole(DomainObject):
 class PackageRole(UserObjectRole):
     protected_object = Package
     name = 'package'
+
+    def __repr__(self):
+        if self.user:
+            return '<%s user="%s" role="%s" package="%s">' % \
+                (self.__class__.__name__, self.user.name, self.role, self.package.name)
+        elif self.authorized_group:
+            return '<%s authorized_group="%s" role="%s" package="%s">' % \
+                (self.__class__.__name__, self.authorized_group.name, self.role, self.package.name)
+        else:
+            assert False, "%s is neither for an authzgroup or for a user" % self.__class__.__name__
+
 protected_objects[PackageRole.protected_object] = PackageRole
 
 class GroupRole(UserObjectRole):
     protected_object = Group
     name = 'group'
+
+    def __repr__(self):
+        if self.user:
+            return '<%s user="%s" role="%s" group="%s">' % \
+                (self.__class__.__name__, self.user.name, self.role, self.group.name)
+        elif self.authorized_group:
+            return '<%s authorized_group="%s" role="%s" group="%s">' % \
+                (self.__class__.__name__, self.authorized_group.name, self.role, self.group.name)
+        else:
+            assert False, "%s is neither for an authzgroup or for a user" % self.__class__.__name__
+
 protected_objects[GroupRole.protected_object] = GroupRole
 
 class AuthorizationGroupRole(UserObjectRole):
     protected_object = AuthorizationGroup
     name = 'authorization_group'
+
+    def __repr__(self):
+        if self.user:
+            return '<%s user="%s" role="%s" authorization_group="%s">' % \
+                (self.__class__.__name__, self.user.name, self.role, self.authorization_group.name)
+        elif self.authorized_group:
+            return '<%s authorized_group="%s" role="%s" authorization_group="%s">' % \
+                (self.__class__.__name__, self.authorized_group.name, self.role, self.authorization_group.name)
+        else:
+            assert False, "%s is neither for an authzgroup or for a user" % self.__class__.__name__
+
 protected_objects[AuthorizationGroupRole.protected_object] = AuthorizationGroupRole
 
 class SystemRole(UserObjectRole):
@@ -355,7 +389,7 @@ def give_all_packages_default_user_roles():
         print 'Creating default user for for %s with admins %s' % (pkg.name, admins)
         setup_default_user_roles(pkg, admins)
 
-# default user roles - used when the config doesn't specify them
+# default user roles - used when the config doesn\'t specify them
 default_default_user_roles = {
     'Package': {"visitor": ["editor"], "logged_in": ["editor"]},
     'Group': {"visitor": ["reader"], "logged_in": ["reader"]},
