@@ -1,4 +1,4 @@
-from datetime import datetime
+import datetime
 
 from meta import *
 from core import *
@@ -14,7 +14,7 @@ rating_table = Table('rating', metadata,
                      Column('user_ip_address', UnicodeText), # alternative to user_id if not logged in
                      Column('package_id', UnicodeText, ForeignKey('package.id')),
                      Column('rating', Float),
-                     Column('created', DateTime, default=datetime.now),
+                     Column('created', DateTime, default=datetime.datetime.now),
                      )
 
 class Rating(DomainObject):
