@@ -9,11 +9,13 @@ __all__ = [
     'IMapper', 'ISession',
     'IMiddleware',
     'IAuthFunctions',
-    'IDomainObjectModification', 'IGroupController', 
+    'IDomainObjectModification', 'IGroupController',
     'IPackageController', 'IPluginObserver',
     'IConfigurable', 'IConfigurer', 'IAuthorizer',
     'IActions', 'IResourceUrlChange', 'IDatasetForm',
     'IGroupForm',
+    'ITagController',
+    'ITemplateHelpers',
 ]
 
 from inspect import isclass
@@ -78,7 +80,7 @@ class IRoutes(Interface):
 
     def after_map(self, map):
         """
-        Called after routes map is set up. ``after_map`` can be used to add fall-back handlers. 
+        Called after routes map is set up. ``after_map`` can be used to add fall-back handlers.
 
         :param map: Routes map object
         :returns: Modified version of the map object
@@ -119,12 +121,12 @@ class IMapper(Interface):
         """
         Receive an object instance after that instance is INSERTed.
         """
-                                     
+
     def after_update(self, mapper, connection, instance):
         """
         Receive an object instance after that instance is UPDATEed.
         """
-    
+
     def after_delete(self, mapper, connection, instance):
         """
         Receive an object instance after that instance is DELETEed.
@@ -181,12 +183,27 @@ class IResourceUrlChange(Interface):
     def notify(self, resource):
         pass
 
+class ITagController(Interface):
+    '''
+    Hook into the Tag controller. These will usually be called just before
+    committing or returning the respective object, i.e. all validation,
+    synchronization and authorization setup are complete.
+
+    '''
+    def before_view(self, tag_dict):
+        '''
+        Extensions will recieve this before the tag gets displayed. The
+        dictionary passed will be the one that gets sent to the template.
+
+        '''
+        return tag_dict
+
 class IGroupController(Interface):
     """
-    Hook into the Group controller. These will 
+    Hook into the Group controller. These will
     usually be called just before committing or returning the
-    respective object, i.e. all validation, synchronization 
-    and authorization setup are complete. 
+    respective object, i.e. all validation, synchronization
+    and authorization setup are complete.
     """
 
     def read(self, entity):
@@ -200,13 +217,20 @@ class IGroupController(Interface):
 
     def authz_add_role(self, object_role):
         pass
-    
+
     def authz_remove_role(self, object_role):
         pass
 
     def delete(self, entity):
         pass
 
+    def before_view(self, pkg_dict):
+        '''
+             Extensions will recieve this before the group gets displayed. The dictionary
+             passed will be the one that gets sent to the template.
+        '''
+        return pkg_dict
+
 class IPackageController(Interface):
     """
     Hook into the package controller.
@@ -224,7 +248,7 @@ class IPackageController(Interface):
 
     def authz_add_role(self, object_role):
         pass
-    
+
     def authz_remove_role(self, object_role):
         pass
 
@@ -264,13 +288,20 @@ class IPackageController(Interface):
 
     def before_index(self, pkg_dict):
         '''
-             Extensions will recieve what will be given to the solr for indexing.
+             Extensions will receive what will be given to the solr for indexing.
              This is essentially a flattened dict (except for multlivlaued fields such as tags
              of all the terms sent to the indexer.  The extension can modify this by returning
              an altered version.
         '''
         return pkg_dict
-        
+
+    def before_view(self, pkg_dict):
+        '''
+             Extensions will recieve this before the dataset gets displayed. The dictionary
+             passed will be the one that gets sent to the template.
+        '''
+        return pkg_dict
+
 
 class IPluginObserver(Interface):
     """
@@ -301,26 +332,26 @@ class IPluginObserver(Interface):
         This method is passed the instantiated service object.
         """
 
-class IConfigurable(Interface): 
+class IConfigurable(Interface):
     """
     Pass configuration to plugins and extensions
     """
-    
+
     def configure(self, config):
         """
         Called by load_environment
         """
 
-class IConfigurer(Interface): 
+class IConfigurer(Interface):
     """
     Configure CKAN (pylons) environment via the ``pylons.config`` object
     """
-    
+
     def update_config(self, config):
         """
         Called by load_environment at earliest point when config is
         available to plugins. The config should be updated in place.
-        
+
         :param config: ``pylons.config`` object
         """
 
@@ -350,14 +381,14 @@ class IAuthorizer(Interface):
         Should return True or False.  A value of False will allow
         other Authorizers to run; True will shortcircuit and return.
         """
-        
+
 class IActions(Interface):
     """
     Allow adding of actions to the logic layer.
     """
     def get_actions(self):
         """
-        Should return a dict, the keys being the name of the logic 
+        Should return a dict, the keys being the name of the logic
         function and the values being the functions themselves.
         """
 
@@ -371,6 +402,16 @@ class IAuthFunctions(Interface):
         implementation overrides
         """
 
+class ITemplateHelpers(Interface):
+    """
+    Allow adding extra template functions available via h variable
+    """
+    def get_helpers(self):
+        """
+        Should return a dict, the keys being the name of the helper
+        function and the values being the functions themselves.
+        """
+
 class IDatasetForm(Interface):
     """
     Allows customisation of the package controller as a plugin.
@@ -380,7 +421,7 @@ class IDatasetForm(Interface):
      - package_form(self)
      - form_to_db_schema(self)
      - db_to_form_schema(self)
-     - check_data_dict(self, data_dict)
+     - check_data_dict(self, data_dict, schema=None)
      - setup_template_variables(self, context, data_dict)
 
     Furthermore, there can be many implementations of this plugin registered
@@ -394,7 +435,7 @@ class IDatasetForm(Interface):
      - package_types(self)
 
     Implementations might want to consider mixing in
-    ckan.controllers.package.DefaultPluggablePackageController which provides
+    ckan.lib.plugins.DefaultDatasetForm which provides
     default behaviours for the 5 method hooks.
 
     """
@@ -409,8 +450,7 @@ class IDatasetForm(Interface):
         There must be exactly one fallback controller defined, any attempt to
         register more than one will throw an exception at startup.  If there's
         no fallback registered at startup the
-        ckan.controllers.package.DefaultPluggablePackageController is used
-        as the fallback.
+        ckan.lib.plugins.DefaultDatasetForm is used as the fallback.
         """
 
     def package_types(self):
@@ -429,6 +469,37 @@ class IDatasetForm(Interface):
 
     ##### Hooks for customising the PackageController's behaviour        #####
     ##### TODO: flesh out the docstrings a little more.                  #####
+    def new_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the new page
+        """
+
+    def comments_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the comments page
+        """
+
+    def search_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the search page (if present)
+        """
+
+    def read_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the read page
+        """
+
+    def history_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the history page
+        """
+
+
     def package_form(self):
         """
         Returns a string representing the location of the template to be
@@ -447,7 +518,7 @@ class IDatasetForm(Interface):
         format suitable for the form (optional)
         """
 
-    def check_data_dict(self, data_dict):
+    def check_data_dict(self, data_dict, schema=None):
         """
         Check if the return data is correct.
 
@@ -485,7 +556,7 @@ class IGroupForm(Interface):
      - package_types(self)
 
     Implementations might want to consider mixing in
-    ckan.controllers.package.DefaultPluggablePackageController which provides
+    ckan.lib.plugins.DefaultGroupForm which provides
     default behaviours for the 5 method hooks.
 
     """
@@ -500,8 +571,7 @@ class IGroupForm(Interface):
         There must be exactly one fallback controller defined, any attempt to
         register more than one will throw an exception at startup.  If there's
         no fallback registered at startup the
-        ckan.controllers.group.DefaultPluggableGroupController is used
-        as the fallback.
+        ckan.lib.plugins.DefaultGroupForm used as the fallback.
         """
 
     def group_types(self):
@@ -520,7 +590,34 @@ class IGroupForm(Interface):
 
     ##### Hooks for customising the PackageController's behaviour        #####
     ##### TODO: flesh out the docstrings a little more.                  #####
-    
+    def new_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the 'new' page. Uses the default_group_type configuration
+        option to determine which plugin to use the template from.
+        """
+
+    def index_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the index page. Uses the default_group_type configuration
+        option to determine which plugin to use the template from.
+        """
+
+    def read_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the read page
+        """
+
+    def history_template(self):
+        """
+        Returns a string representing the location of the template to be
+        rendered for the history page
+        """
+
+
+
     def package_form(self):
         """
         Returns a string representing the location of the template to be
