@@ -9,7 +9,7 @@ class TestAdminController(WsgiAppCase):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
 
     #test that only sysadmins can access the /ckan-admin page
     def test_index(self):
@@ -46,7 +46,7 @@ class TestAdminAuthzController(WsgiAppCase):
 
     @classmethod
     def teardown_class(self):
-        CreateTestData.delete()
+        model.repo.rebuild_db()
 
     def test_role_table(self):
 
@@ -334,7 +334,7 @@ class TestAdminTrashController(WsgiAppCase):
         url = url_for('ckanadmin', action='trash')
         res = self.app.get(url, extra_environ=as_testsysadmin)
         form = res.forms['undelete-'+rev.id]
-        res = form.submit('submit', status=[302], extra_environ=as_testsysadmin)
+        res = form.submit('action', status=[302], extra_environ=as_testsysadmin)
         res = res.follow(extra_environ=as_testsysadmin)
 
         assert 'Revision updated' in res
