@@ -1,8 +1,9 @@
 from ckan.tests import *
 import ckan.model as model
 from ckan.lib.create_test_data import CreateTestData
+from base import FunctionalTestCase
 
-class TestRelationships(TestController):
+class TestRelationships(FunctionalTestCase):
     @classmethod
     def setup_class(self):
         create = CreateTestData
@@ -17,11 +18,11 @@ class TestRelationships(TestController):
             offset = url_for(controller='package', action='read', id=pkg_name)
             res = self.app.get(offset)
             pkg = model.Package.by_name(pkg_name)
-            assert '%s - Data Packages' % pkg.title in res
+            assert '%s - Datasets' % pkg.title in res
             return res
         res = read_package(u'homer')
         self.check_named_element(res, 'li', 'is a child of', 'abraham')
-        self.check_named_element(res, 'li', 'is a child of', '<a href="/package/abraham">abraham</a>')
+        self.check_named_element(res, 'li', 'is a child of', '<a href="/dataset/abraham">abraham</a>')
         self.check_named_element(res, 'li', 'is a parent of', 'bart')
         self.check_named_element(res, 'li', 'is a parent of', 'lisa')
         self.check_named_element(res, 'li', 'has derivation', 'homer_derived')
