@@ -19,7 +19,7 @@ class TestPackage:
         self.pkg1 = model.Package(name=self.name)
         model.Session.add(self.pkg1)
         self.pkg1.notes = self.notes
-        self.pkg1.license_id = u'gpl-3.0'
+        self.pkg1.license_id = u'odc-by'
         model.Session.commit()
         model.Session.remove()
 
@@ -71,8 +71,8 @@ class TestPackage:
         package = model.Package.by_name(self.name)
         assert package.name == self.name
         assert package.notes == self.notes
-        assert package.license.id == u'gpl-3.0'
-        assert package.license.title == u'OSI Approved::GNU General Public License version 3.0 (GPLv3)', package.license.title
+        assert package.license.id == u'odc-by'
+        assert package.license.title == u'Open Data Commons Attribution License', package.license.title
 
     def test_update_package(self):
         newnotes = u'Written by Beethoven'
@@ -105,7 +105,7 @@ class TestPackage:
         assert out['name'] == pkg.name
         assert out['license'] == pkg.license.title
         assert out['license_id'] == pkg.license.id
-        assert out['tags'] == [tag.name for tag in pkg.tags]
+        assert out['tags'] == [tag.name for tag in pkg.get_tags()]
         assert out['metadata_modified'] == pkg.metadata_modified.isoformat()
         assert out['metadata_created'] == pkg.metadata_created.isoformat()
         assert_equal(out['notes'], pkg.notes)
@@ -212,7 +212,7 @@ class TestPackageWithTags:
         self.tag = model.Tag(name=self.tagname)
         self.tag2 = model.Tag(name=self.tagname2)
         pkg2tag = model.PackageTag(package=pkg, tag=self.tag)
-        pkg.tags.append(self.tag2)
+        pkg.add_tag(self.tag2)
         model.Session.add_all([pkg,self.tag,self.tag2,pkg2tag])
         model.Session.commit()
         self.pkg2tag_id = pkg2tag.id
@@ -224,7 +224,7 @@ class TestPackageWithTags:
 
     def test_1(self):
         pkg = model.Package.by_name(self.pkgname)
-        assert len(pkg.tags) == 2
+        assert len(pkg.get_tags()) == 2
         # pkg2tag = model.Session.query(model.PackageTag).get(self.pkg2tag_id)
         # assert pkg2tag.package.name == self.pkgname
 
@@ -245,7 +245,7 @@ class TestPackageWithTags:
         except AttributeError: # sqlalchemy 0.4
             model.Session.clear()
         outpkg = model.Package.by_name(self.pkgname)
-        assert len(outpkg.tags) == 3
+        assert len(outpkg.get_tags()) == 3
         t1 = model.Tag.by_name(self.tagname)
         assert len(t1.package_tags) == 1
 
@@ -255,9 +255,9 @@ class TestPackageWithTags:
         except AttributeError: # sqlalchemy 0.4
             model.Session.clear()
         pkg = model.Package.by_name(self.pkgname)
-        assert len(pkg.tags) == 3
+        assert len(pkg.get_tags()) == 3, len(pkg.get_tags())
         pkg.add_tag_by_name(self.tagname)
-        assert len(pkg.tags) == 3
+        assert len(pkg.get_tags()) == 3
 
 
 class TestPackageTagSearch:
@@ -271,20 +271,18 @@ class TestPackageTagSearch:
         self.tagname = u'russian-tag-we-will-delete'
         tag3 = model.Tag(name=self.tagname)
         pkg = model.Package.by_name(u'annakarenina')
-        pkg.tags.append(tag3)
+        pkg.add_tag(tag3)
         model.repo.commit_and_remove()
 
         model.repo.new_revision()
         pkg = model.Package.by_name(u'annakarenina')
-        # we aren't guaranteed it is last ...
-        idx = [ t.name for t in pkg.tags].index(self.tagname)
-        del pkg.tags[idx]
+        pkg.remove_tag(tag3)
         # now do a tag for ordering
         tagordered = model.Tag(name=self.orderedfirst)
         wap = model.Package.by_name(u'warandpeace')
         # do them the wrong way round
-        tagordered.packages.append(wap)
-        tagordered.packages.append(pkg)
+        wap.add_tag(tagordered)
+        pkg.add_tag(tagordered)
         model.repo.commit_and_remove()
 
     @classmethod 
@@ -295,7 +293,7 @@ class TestPackageTagSearch:
     def test_0_deleted_package_tags(self):
         pkg = model.Package.by_name(u'annakarenina')
         tag = model.Tag.by_name(self.tagname)
-        assert len(pkg.tags) == 4
+        assert len(pkg.get_tags()) == 4, len(pkg.get_tags())
         assert len(tag.packages) == 0
 
     def test_1_tag_search_1(self):
@@ -313,9 +311,9 @@ class TestPackageTagSearch:
     
     def test_alphabetical_ordering(self):
         pkg = model.Package.by_name(u'annakarenina')
-        tag = pkg.tags_ordered[0]
-        assert tag.name == self.orderedfirst, pkg.tags
-        assert tag.packages_ordered[0].name == 'annakarenina', tag.packages
+        tag = pkg.get_tags()[0]
+        assert tag.name == self.orderedfirst
+        assert tag.packages[0].name == 'annakarenina', tag.packages
 
 
 class TestPackageRevisions:
@@ -481,6 +479,9 @@ class TestPackagePurge:
     @classmethod
     def setup_class(self):
         CreateTestData.create()
+    @classmethod
+    def teardown_class(self):
+        model.repo.rebuild_db()
     def test_purge(self):
         pkgs = model.Session.query(model.Package).all()
         for p in pkgs:
